"""
===========================================
COBalD - the Opportunistic Balancing Daemon
===========================================

.. image:: https://readthedocs.org/projects/cobald/badge/?version=latest
    :target: http://cobald.readthedocs.io/en/latest/?badge=latest
    :alt: Documentation Status

This is a **draft** for a feedback based balancing system for providing opportunistic resources.
"""
__title__ = 'cobald'
__summary__ = 'COBalD - the Opportunistic Balancing Deamon'
__url__ = 'https://github.com/MaineKuehn/cobald'

__version__ = '0.9.0'
__author__ = 'Eileen Kuehn, Max Fischer'
__email__ = 'mainekuehn@gmail.com'
__copyright__ = '2018 %s' % __author__
__keywords__ = 'opportunistic scheduling scheduler demand feedback-loop cobald'
